//convert-GraphGeod-MPI.c 
/*
   A MPI program that converts the snapshot organized .GraphGeod files
	from ChemNetworks to a set of molecule ID organized files      
 
   input(s): *.GraphGeod
   output(s): mol*.GG
 
   Note: cannot have any atom/molecule labels be zero

   The terms:
	input-name
	final-output-name
	intermediate-name
 
	are special markers to allow a bash script to find and change those names

   Printed errors:
	er1:		ntasks out of range
	er2:		nmol out of range
	er3, er4:	memory allocation error: ptnrData
	er5:		subConvert cannot open the *.GraphGeod file
	er6:		subConvert failed to run correctly
	er7:		catFile cannot open intermediate file
	er8, er10, er12:catFile failed to run correctly
	er9, er11, er13:failed to remove intermediate file


   last modified: March 2016 by L. Edens
 */

#define FLN 10000	//maximum string length for each line
#define MXPTNRN 50      //maximum number of patern data per mol per snapshot-> max possible connections * 3
//#define MXSNAP 100000 //maximum number of snapshots   

#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<math.h>
#include "/opt/openmpi/include/mpi.h"

int subConvert(int ssnap, int nsnap, int nmol, int rank);
int catFile(char* infile, FILE *out);

int main(int argc, char **argv)
{
	char infile[FLN];
	char outfile[FLN];
	FILE *in, *out;

	int tsnap = 1000;	//total number of snapshots
	int nmol = 216;		//total number of molecules

	int ntasks=0;		//total number of threads -> will be set by MPI_Comm_size
	int rank=0;		//thread number -> will be set by MPI_Comm_rank
	int nsnap=0;		//number of snapshots for each rank
	int ssnap=0;		//staring snapshot number
	int fsnap=0;		//final set of snapshots to accound for uneven numbers
	int rmol=0;		//number of molecules for each rank
	int frmol=0;		//final set of molecules for each rank
	int i, j, er;

	//setup MPI
	MPI_Init(&argc, &argv);	
	MPI_Comm_size(MPI_COMM_WORLD, &ntasks);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);

	//printf("Input:%d  %s",argc,argv[2]);
	//printf("Number of threads = %d, My rank = %d, Hostname = %s\n", nTasks, rank, hostname);

	//check for input errors
	if(tsnap <= 0){
		if(rank==0){printf("er1\n");}
		exit(1);
	}
	if(nmol <= 0){
		if(rank==0){printf("er2\n");}
		exit(1);
	}
	//determine snapshot split
	if(ntasks < tsnap){
		//determine split in snapshots
		nsnap = floor(tsnap/(ntasks));
		if((nsnap*ntasks)==tsnap){fsnap = nsnap;}
		else {fsnap = tsnap-(nsnap*(ntasks-1));}
	}
	else{ //ntasks >= tsnap
		//match one timestep to one thread
		nsnap = 1;
		fsnap = 1;
	}

	//run subroutine
	if(rank == 0){er=subConvert(1, nsnap, nmol, rank); }
	else if(rank == ntasks-1 ){er=subConvert((nsnap*rank+1), fsnap, nmol, rank); }
	else if(rank >= tsnap){er=0;}
	else {er=subConvert((nsnap*rank+1), nsnap, nmol, rank);}

	if(er != 0){printf("er6\n");exit(1);}

	//wait for all threads to finish
        MPI_Barrier(MPI_COMM_WORLD);

	//concatenate outputs
	//an efficient method is to split up the intermediate files according to nmol (as opposed to nsnaps like above)
	//each rank now handles: mol#.GG = mol#.0.GG + mol#.1.GG + ...mol#.ntasks.GG, where # is split across the ranks
	if(ntasks < nmol)
	{
		//determine mol split
		rmol = floor(nmol/(ntasks));
		if((rmol*ntasks)==nmol){frmol = rmol;}
		else {frmol = nmol-(rmol*(ntasks-1));}

		if(rank != ntasks-1)
		{
			//loop over the molecular IDs assiged to this rank
			for(i=1;i<=rmol;i++)
			{
	/*final-output-name*/   sprintf(outfile,"mol%d.GG",(rank*rmol)+i);
				//this is the best way to clear any existing files
				out = fopen(outfile,"w");
				fclose(out);
				out = fopen(outfile,"a");	

				for(j=0;j<ntasks;j++)
				{
		/*intermediate-name*/   sprintf(infile,"mol%d.%d.GG",(rank*rmol)+i,j);

					er = catFile(infile, out);				
					if(er != 0) {printf("er8\n");}			

					er = remove(infile);
					if(er != 0) {printf("er9\n");}
				}
				fclose(out);
			}
		}
		else //special case for the last rank
		{
			//copied from above
			for(i=1;i<=frmol;i++)
			{
	/*final-output-name*/   sprintf(outfile,"mol%d.GG",(rank*rmol)+i);   
				out = fopen(outfile,"w");
				fclose(out);
				out = fopen(outfile,"a");	

				for(j=0;j<ntasks;j++)
				{
		/*intermediate-name*/   sprintf(infile,"mol%d.%d.GG",(rank*rmol)+i,j);
                                        er = catFile(infile, out);
                                        if(er != 0) {printf("er10\n");}
			
					er = remove(infile);
					if(er != 0) {printf("er11\n");}
				}
				fclose(out);
			}
		}
	}
	else //ntasks >= nmol
	{
		if(rank+1 <= nmol) //match 1 rank to 1 mol and ignore higher ranks
		{
/*final-output-name*/   sprintf(outfile,"mol%d.GG",rank+1);
			out = fopen(outfile,"w");
			fclose(out);
			out = fopen(outfile,"a");	

			for(j=0;j<ntasks;j++)
			{
	/*intermediate-name*///   sprintf(infile,"mol%d.%d.GG",rank+1,j);
				er = catFile(infile, out);
				if(er != 0) {printf("er12\n");}

				er = remove(infile);
				if(er != 0) {printf("er13\n");}		
			}
			fclose(out);
		}
	}
	MPI_Finalize();
	return (0);
}

int subConvert(int ssnap, int nsnap, int nmol, int rank)
{

        char file[FLN];
        FILE *in, *out;
        int i, j, skip, temp1, temp2;
        int mol1, mol2, ptnr;
        int **ptnrData;
        char buffer[100];
        int pbcx,pbcy,pbcz,atm1,atm2;


        //ptnrData: (molecule #)(data1 data1 data1 data2 data2 data2 ... ) 
        //each line has a variable length filled in        
        
        ptnrData = (int **)calloc(nmol, sizeof(int *));
        if(ptnrData==NULL){printf("er3\n"); exit(1);}		
        *ptnrData = (int *)calloc(nmol*MXPTNRN, sizeof(int));
        if(*ptnrData==NULL){printf("er4\n"); exit(1);}
        for(i=1; i<nmol; i++){ptnrData[i]=ptnrData[i-1]+MXPTNRN;}
        //if(rank==0){printf("memory allocation done check:%d\n", ptnrData[4][20]);}
        
        //create/clear files for all molecules
       for(i=0; i<nmol; i++)
       {
		//if name is changed it needs to be changed below as well
/*intermediate-name*/   sprintf(file,"mol%d.%d.GG",i+1, rank);
                out=fopen(file,"w");
		//fprintf(out,"ssnap:%d \t nsnap:%d\n",ssnap,nsnap);
                fclose(out);
        }

        for(i=ssnap; i<(nsnap+ssnap); i++)
        {
		//read in original file
 /*input-name*/   sprintf(file, "Input.water%d.xyz.water%d.xyz.GraphGeod",i,i);

                if((in=fopen(file,"r"))==NULL)
                {
                        printf("er5:%s\n",file);
                        fclose(in);
                        exit(1);
                }

                //read in ith snapshot file and store in ptnrData array
		ptnr = 0;
		temp1=temp2=0;
               while(fscanf(in, "%d %d %d %d %d %d %d", &mol1,&mol2,&pbcx,&pbcy,&pbcz,&atm1,&atm2)!=EOF) // read two molecule ids
                {
                        fgets(buffer,sizeof(buffer),in);
		
                        //sorted into mol#-1 to account for 0th element
                        skip = 0;
                        while(ptnrData[mol1-1][skip] != 0){skip +=3;}

                        ptnrData[mol1-1][0+skip]= mol2;
                        ptnrData[mol1-1][1+skip]= atm1;
                        ptnrData[mol1-1][2+skip]= atm2;

                        //both pair directions must be counted
                        skip = 0;
                        while(ptnrData[mol2-1][skip] != 0){skip +=3;}
                        ptnrData[mol2-1][0+skip]= mol1;
                        ptnrData[mol2-1][1+skip]= atm2;
                        ptnrData[mol2-1][2+skip]= atm1;

                }
                fclose(in);
                //if(rank==0){printf("read in done\n");}

		//sort ptnrData via output
		for(j=0; j<nmol; j++)
                {
                        if(ptnrData[j][0] != 0)
                       {
        /*intermediate-name*/   sprintf(file,"mol%d.%d.GG",j+1,rank);
                                out=fopen(file,"a");

                                skip = 0;
                                while(ptnrData[j][skip] != 0)
                                {
                                        fprintf(out,"%d %d %d %d \n",i,ptnrData[j][0+skip],ptnrData[j][1+skip],ptnrData[j][2+skip]);

					//erase date after print to prep for next loop
					ptnrData[j][0+skip]= ptnrData[j][1+skip]= ptnrData[j][2+skip] = 0;
                                        skip += 3;
                                }
                                fclose(out);
                        }
                }
        }
        free(*ptnrData);
        free(ptnrData);
        return(0);
}

int catFile(char* infile, FILE *out)
{
        char *buffer;
        FILE *in;
        size_t nread;

        if((in=fopen(infile,"r"))==NULL)
        {
                printf("er7\n");
                fclose(in);
                exit(0);
        }

        //make sure the pointer is at the beginning of file
        fseek(in, 0L, SEEK_SET);
       
	//copy the file (as a string) in 16MB chunks
	buffer = (char*) malloc(16000 * sizeof(char));
        while((nread = fread(buffer,sizeof(char),16000,in)) > 0)
        	{fwrite(buffer,sizeof(char),nread,out);}       
 
	fclose(in);
	return(0);
}
