#!/bin/bash
#**************************************
#control script for the lifetime correction2 method
#
#This script uses mpicc to compile convert-GraphGeod-MPI.c 
#This script uses a .PBS submission script 
#
# Requires: .GraphGeod outputs from ChemNetworks
#            input file (input-corr)
#            conversion C file (convert-GraphGeod-MPI.c)
#            lifetime correction executable (lifetime_corr2.exe)
#	     lifetime submission script (lifetimes.pbs)
#            average lifetime counter executable (count_lifetime2.exe)
#
# Steps:
# 1. Alters input-corr file according user inputs
# 2. Alters and recompiles (via mpicc) conversion C file
# 3. Creates and runs a PBS submission script named convert.PBS
# 4. Runs lifetime correction (twice using mode 0, then mode 3)
# 5. Moves the old .GG files into a subfolder and renames the new GG files
# 6. Runs lifetime correction again (mode 0)
# 7. Outputs lifetime results to averages.txt
#
#
# 
#
#
#made by L.Edens
#last modified: April 2016
#**************************************

echo "Start  " `date`
echo ''

#**********
#Default inputs: 
#Optional command line parameters follow the same order


#number of compute nodes for the conversion program
cnodes=1
	
#number of processors per node or the conversion program
ppn=1

#Input control filename
inputFile="input-corr"

#number of molecules (called nodes in input-corr)
mol=$(awk -F'] ' '$1=="[NUMBER OF NODES"{print $2;exit;}' $inputFile)

#number of snapshots
snaps=$(awk -F'] ' '$1=="[NUMBER OF SNAPS"{print $2;exit;}' $inputFile)

#erase previous output files (No=0/Yes=1)
erase=0

#convert files (No=0/Yes=1)
convert=1

#lifetime correction (No=0/Yes=1)
lifetime=1

#Input .GraphGeod files: replace numbers with %d
GraphGeod="water.input.water%d.xyz.water%d.xyz.GraphGeod"

#Number of %d in .GraphGeod filename
count=2

#**********
#Other inputs (cannot be changed on the command line)

#Conversion C code filename (not the executable)
convertFile="convert-GraphGeod-MPI.c"

#Conversion executable
convertexe="convert.exe"

#Conversion PBS submission script
convertpbs="convert.pbs"

#Lifetime correction executable filename (not the code files)
lifetimeFile="lifetime_corr2.exe"

#Lifetime correction PBS submission script
lifetimepbs="lifetime.pbs"

#Average lifetime weighted histogram executable
lifetimeCount="count_lifetime2.exe"

#Name of an example converted file for use in error checking
#the filename style can be changed in the conversion and lifetime correction C codes 
errorGG="mol1.GG"

#**********Step 1
#reset input vlaues if entered on the command line
if [ $# -gt 0 ] && [ $# -lt 11 ]
then
   if [ $# -ge 1 ]; then
      cnodes=$1
   fi
   if [ $# -ge 2 ]; then
      ppn=$2
   fi
   if [ $# -ge 3 ]; then
      inputFile=$3
   fi
   if [ $# -ge 4 ]; then
      mol=$4
      #modify inputFile with new number

      #find current file number
      oldn=$(awk -F'] ' '$1=="[NUMBER OF NODES"{print $2;exit;}' $inputFile)

      #update with new file number
      oldnp=$(($oldn+1))
      molp=$((mol+1))
      sed -i "s/\[NUMBER OF NODES\] $oldn/\[NUMBER OF NODES\] $mol/" $inputFile
      sed -i "s/\[IDS OF SOLUTE\] $oldnp $oldnp/\[IDS OF SOLUTE\] $molp $molp/" $inputFile
      sed -i "s/\[IDS OF SOLVENT1\] 1 $oldn/\[IDS OF SOLVENT1\] 1 $mol/" $inputFile
      sed -i "s/\[IDS OF SOLVENT2\] $oldnp $oldnp/\[IDS OF SOLVENT2\] $molp $molp/" $inputFile

   fi
   if [ $# -ge 5 ]; then
      snaps=$5
      #modify inputFile with new number

      #find current file number
      oldn=$(awk -F'] ' '$1=="[NUMBER OF SNAPS"{print $2;exit;}' $inputFile)

      #update with new file number
      sed -i "0,/\[NUMBER OF SNAPS\] $oldn/s//\[NUMBER OF SNAPS\] $snaps/" $inputFile

   fi
   if [ $# -ge 6 ]; then
      erase=$6
   fi
   if [ $# -ge 7 ]; then
      convert=$7
   fi
   if [ $# -ge 8 ]; then
      lifetime=$8
   fi
   if [ $# -ge 9 ]; then
      GraphGeod=$9
   fi
   if [ $# == 10 ]; then
      count=${10}
   fi
fi

if [ $# -ge 11 ]; then
   echo "Too many arguments. Using defaut inputs."
fi

#erase previous outputs
if [ $erase == 1 ]
then
   if [ $convert == 1 ]; then
      fext=$(echo $GraphGeod | sed -e 's/\(.*\)\./\1 /' | awk '{print $2}' )
      rm *$fext
   fi
   if [ $lifetime == 1 ]; then
      fext=$(echo $outputFiles | sed -e 's/\(.*\)\./\1 /' | awk '{print $2}' )
      rm *$fext
   fi
fi


#**********Step 2
#Modify convertFile

#number of snapshots
oldn=$(grep -Eo 'int tsnap = [0-9]{1,10}' $convertFile | awk '{print $4}')
sed -i "0,/int tsnap = $oldn/s//int tsnap = $snaps/" $convertFile

#number of molecules
oldn=$(grep -Eo 'int nmol = [0-9]{1,10}' $convertFile | awk '{print $4}')
sed -i "0,/int nmol = $oldn/s//int nmol = $mol/" $convertFile

#input filename
convertGraphGeod="$GraphGeod\""
lnend=",i"

for ((i=0; i<$count; i++)); do
convertGraphGeod="$convertGraphGeod$lnend"
done
sed -i "/*input-name*/c\ /*input-name*/   sprintf(file, \"$convertGraphGeod);" $convertFile


#Recompile convertfile
mpicc $convertFile -lm -o $convertexe


#**********Step 3

#Build a PBS submission script
rm -f $convertpbs
echo "#PBS -N convertGG" > $convertpbs
echo "#PBS -l walltime=200:00:00" >> $convertpbs
echo "#PBS -l nodes=$cnodes:ppn=$ppn" >> $convertpbs
echo '' >> $convertpbs

#"./" is used to not rely on $PATH
echo "set mycommand = \"$convertexe\"" >> $convertpbs
echo '' >> $convertpbs

# count the number of processors assigned by PBS
echo "if (\$?PBS_NODEFILE && \$?PBS_JOBID) then" >> $convertpbs
echo "   set NP = \`wc -l < \$PBS_NODEFILE\`" >> $convertpbs
echo "   echo \"Running on \$NP processors: \"\`cat \$PBS_NODEFILE\`" >> $convertpbs
echo "else" >> $convertpbs
echo "   echo \"This script must be submitted to PBS with 'qsub -l nodes=X'\"" >> $convertpbs
echo "   exit 1" >> $convertpbs
echo "endif" >> $convertpbs
echo '' >> $convertpbs

# set up the job and input files, exiting if anything went wrong
echo "cd \"\$PBS_O_WORKDIR\" || exit 1" >> $convertpbs
echo '' >> $convertpbs

#run command
echo "mpiexec -n \$NP \$PBS_O_WORKDIR/\$mycommand" >> $convertpbs


#Run conversion and wait for it to finish
echo "Conversion start"
JOBID=$(qsub $convertpbs)
while qstat $JOBID &> /dev/null; do
   sleep 30;
done
echo "Conversion end"

#Remove PBS submission script and executable
rm $convertexe
rm $convertpbs
rm convertGG.e*
rm convertGG.o*

#Check to make sure files were created
errorGG=$(wc $errorGG | awk '{print $1} ')
if [ $errorGG -eq 0 ]; then   
   echo "Exiting: Converted files have no data. Check inputs."
   exit
fi

#***********

#Lifetime correction

#**********Step 4
echo "start lifetime calculation"

#make sure the input file is set correctly
oldn=$(awk -F'] ' '$1=="[LIFETIME OF SOLVENT1 SOLVENT1"{print $2;exit;}' $inputFile)
oldl=$(echo $oldn | awk '{print $2}')
oldb=$(echo $oldn | awk '{print $3}')
sed -i "s/\[LIFETIME OF SOLVENT1 SOLVENT1] [0-9] $oldl $oldb/\[LIFETIME OF SOLVENT1 SOLVENT1] 1 1 0/" $inputFile

#Build a PBS submission script
rm -f $lifetimepbs
echo "#PBS -N lifetime" > $lifetimepbs
echo "#PBS -l walltime=200:00:00" >> $lifetimepbs
echo "#PBS -l nodes=1:ppn=1" >> $lifetimepbs
echo '' >> $lifetimepbs
echo "cd \"\$PBS_O_WORKDIR\" || exit 1" >> $lifetimepbs
echo '' >> $lifetimepbs
echo "./$lifetimeFile $inputFile" >> $lifetimepbs

#run lifetime correction
echo "lifetime: step 1"
JOBID=$(qsub $lifetimepbs)
while qstat $JOBID &> /dev/null; do
   sleep 30;
done
echo "end submission"

#find average lifetime and break values
awk '$6-$5+1 >= 0 {print $6-$5+1}' Lifetimes-1-1 > overall-lifetime
./$lifetimeCount overall-lifetime
x=$(awk '$1=="average"{print $6}' lifetime-distri) 
rm overall-lifetime
rm lifetime-distri

awk '$6==2 || $6==3 && $9-$8 >= 0 {print $9-$8}' Switches-1-1 > overall-break
./$lifetimeCount overall-break
y=$(awk '$1=="average"{print $6}' lifetime-distri) 
rm overall-break
rm lifetime-distri

rm Lifetimes-1-1
rm Switches-1-1
rm Degrees-1-1

#update input file
sed -i "s/\[LIFETIME OF SOLVENT1 SOLVENT1] 1 1 0/\[LIFETIME OF SOLVENT1 SOLVENT1] 1 $x $y/" $inputFile

#******
echo "lifetime: step 2"
JOBID=$(qsub $lifetimepbs)
while qstat $JOBID &> /dev/null; do
   sleep 30;
done
echo "end submission"

#find average lifetime and break values
awk '$6-$5+1 >= 0 {print $6-$5+1}' Lifetimes-1-1 > overall-lifetime-orig
./$lifetimeCount overall-lifetime-orig
j=$(awk '$1=="average"{print $6}' lifetime-distri) 
mv Lifetimes-1-1 Lifetimes-1-1-orig 
rm lifetime-distri

awk '$6==2 || $6==3 && $9-$8 >= 0 {print $9-$8}' Switches-1-1 > overall-break-orig
./$lifetimeCount overall-break-orig
k=$(awk '$1=="average"{print $6}' lifetime-distri)
mv Switches-1-1 Switches-1-1-orig
rm lifetime-distri

mv Degrees-1-1 Degrees-1-1-orig

#update input file
sed -i 's/\[MODE] 0/\[MODE] 3/' $inputFile
sed -i "s/\[LIFETIME OF SOLVENT1 SOLVENT1] 1 $x $y/\[LIFETIME OF SOLVENT1 SOLVENT1] 1 $j $k/" $inputFile

#******
echo "lifetime: step 3"
JOBID=$(qsub $lifetimepbs)
while qstat $JOBID &> /dev/null; do
   sleep 30;
done
echo "end submission"


#**********Step 5

echo "moving files"
mkdir uncorrected-GG
mv mol*.GG ./uncorrected-GG

for((i=1; i<=$mol;i++))
do
mv mol$i.GG-updated mol$i.GG
done

#update input file
sed -i 's/\[MODE] 3/\[MODE] 0/' $inputFile


#**********Step 6

echo "lifetime: step 4"
JOBID=$(qsub $lifetimepbs)
while qstat $JOBID &> /dev/null; do
   sleep 30;
done
echo "end submission"

#find average lifetime and break values
awk '$6-$5+1 >= 0 {print $6-$5+1}' Lifetimes-1-1 > overall-lifetime-updt
./$lifetimeCount overall-lifetime-updt
x=$(awk '$1=="average"{print $6}' lifetime-distri) 
mv Lifetimes-1-1 Lifetimes-1-1-updt 
rm lifetime-distri

awk '$6==2 || $6==3 && $9-$8 >= 0 {print $9-$8}' Switches-1-1 > overall-break-updt
./$lifetimeCount overall-break-updt
y=$(awk '$1=="average"{print $6}' lifetime-distri)
mv Switches-1-1 Switches-1-1-updt
rm lifetime-distri

mv Degrees-1-1 Degrees-1-1-updt
rm $lifetimepbs
rm lifetime.e*
rm lifetime.o*

#**********Step 7

echo "uncorrected average lifetime: $j snaps" >> averages.txt
echo "uncorrected average break: $k snaps" >> averages.txt
echo "corrected average lifetime: $x snaps" >> averages.txt
echo "corrected average break: $y snaps" >> averages.txt
echo ''
echo "Done   " `date`
